# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 01:50:10 2020

@author: Georg Ganzenmueller, Fraunhofer Ernst-Mach Institut, EMI

This code is performs true strain analysis for cylindric specimens based on the 
evaluation of shadow images.
The evolution of the minimal diameter is tracked. Triaxiality of stress is
evaluated using Bridgman's correction [1].
The algorithm works by determining the silhouette of the specimen gauge region
using Canny-edge detection. The minimum distance between the edges constitutes
the current cylindrical diameter. In the vicinity of the minimum location,
the local edge curvature is calculated, which is then used by the Bridgman
correctiion, providing a measure of stress triaxiality.

This file is the user interface. The actual silhouette detection is performed
in the class SilhouetteDiameter.

The user needs to provide a path to a folder containing the images to be analysed,
along with their file type suffix. The images need to be labeled with increasing numbers.
The specimen diameter should be resolved with at least 100 pixels in order to obtain accurate results
with strain errors < 1%.

Two parameters significantly affect the accuracy of the strain computation, and especially the accuracy
of the stress triaxiality factor: The amount of smoothing (nsmooth) applied to the specimen silhouette
detected by the Canny-edge filter, and the order of its spline representation. Good values
for nsmooth are 100 -- 1000, and order = 1 to 3.



[1] P. W. Bridgman, Studies in Large Plastic Flow and Fracture, McGraw-Hill, New York, (1952).
"""

import configparser, triaxiality

cfg = configparser.ConfigParser()
cfg.read("constrain.ini")
suffix = cfg.get("DEFAULT", "suffix")
nMax = cfg.getint("DEFAULT", "nMax")
transpose = cfg.getboolean("DEFAULT", "transpose") # rotate image? Specimen must be vertical in image
notched_specimen = cfg.getboolean("DEFAULT", "notched_specimen")
nueps = cfg.getfloat("DEFAULT", "nueps")

###############################################################################
# *********** Only edit lines above *************
###############################################################################

import numpy as np
import matplotlib.pyplot as plt
import glob
from Silhouette import Silhouette

def read_images():
    filenames = []
    for file in glob.glob(suffix):
        filenames.append(file)
        
    from natsort import natsorted
    filenames = natsorted(filenames)
        
    if nMax > 0:
        return filenames[:nMax]
    else:
        return filenames

filenames = read_images()
N = len(filenames)

diameters = np.zeros(N)
notch_radii = np.zeros(N)
valid_flag = np.zeros(N, dtype=bool)

for i in range(0,len(filenames)):
    print("\n-------------- file: %s --------------" %(filenames[i]))
    sil = Silhouette(filenames[i], plot=False, transpose=transpose)
                             
    if sil.status == True:
        if sil.diameter > 0:
            diameters[i] = sil.diameter
            if sil.curvature_valid:
                valid_flag[i] = True
                notch_radii[i] = sil.notch_radius
                print("current diameter in pixels is: ", sil.diameter, " notch radius is; ", sil.notch_radius, "sigma is", sil.notch_sigma)
            notch_radii[i] = max(notch_radii[i], 0.25*diameters[i]) # limit severe triaxiality to 1.3
                
    else:
        print("something went wrong in the analysis, exiting prematurely")
        break

eps_eq = (1./nueps) * np.log(diameters[0]/diameters) # equivalent true strain

savedata = np.column_stack((np.arange(len(diameters)), eps_eq, diameters, notch_radii))
np.savetxt("strain.txt",savedata, header="index, eps_eq, diameter, notch_radius")


plt.style.use('dark_background')
fig, axs = plt.subplots(2,2)

# --- TOP LEFT plot: strain vs. frame number ---
ax = axs[0,0]
color = 'tab:red'
ax.set_xlabel('frame No.')
ax.set_ylabel('equivalent true strain')
ax.plot(eps_eq, color=color, label="strain")

# --- BOTTOM LEFT plt: diamater vs frame number ---
ax = axs[1,0]
ax.set_xlabel('frame No.')
ax.set_ylabel('diameter [px]')
ax.plot(diameters, "rx", label="diameter")

# --- TOP RIGHT plot: stress triaxiality vs strain ---
ax = axs[0,1]
ax.set_xlabel('eq. true strain')
ax.set_ylabel('stress triaxiality [-]')
ax.plot(eps_eq,triaxiality.triax_function(diameters, notch_radii), "bo", label="stress triax.")
ax.legend()

# --- BOTTOM RIGHT plot: Bridgman factor vs strain ---
ax = axs[1,1]
ax.set_xlabel('eq. true strain')
ax.set_ylabel('Bridgman factor [-]')
ax.plot(eps_eq, triaxiality.bridgman_function(diameters, notch_radii), "go", label="Bridgman factor")
ax.legend()

fig.tight_layout()  # otherwise the right y-label is slightly clipped
plt.show()



# if len(filenames) > 2:
#     def powerlaw(x, a, n, offset):
#         return a * (np.abs(x)**n) + offset
#     def powerlaw_unnotched(x, a, n):
#         return a * (np.abs(x)**n)
    
#     # need a data subset which contains only strain and ratios if valid_flag is set to true
#     eps_eq_sub = np.compress(valid_flag, eps_eq)
#     ratios_sub = np.compress(valid_flag, ratios)
#     #print("length of original array: ", len(eps_eq))
#     #print("length of subset array: ", len(eps_eq_sub))
#     #print(eps_eq_sub)
    
#     if notched_specimen:
#         try:
#             p0 = [1, 0.5, 0.5]
#             popt, pcov = curve_fit(powerlaw, eps_eq_sub, ratios_sub, p0=p0)
#             ratio_fit = powerlaw(eps_eq, *popt)
#         except:
#             ratio_fit = ratios
            
#     else:
#         try:
#             p0 = [1, 0.5]
#             popt, pcov = curve_fit(powerlaw_unnotched, eps_eq_sub, ratios_sub, p0=p0)
#             ratio_fit = powerlaw_unnotched(eps_eq, *popt)
#         except:
#             ratio_fit = ratios
    
#     # now replace invalid ratios with fit
    
#     #print("original ratios", ratios)
#     ratios = np.where(valid_flag, ratios, ratio_fit)
#     #print("repaired ratios", ratios)
      
    
#     bridgman_fit = bridgman_function(ratio_fit)
#     triax_fit = triax_function(ratio_fit)

#     # compute Bridgman factor and stress triaxiality
#     bridgmans = bridgman_function(ratios)
#     triaxs = triax_function(ratios)    
    
# else:
#     bridgman_fit = bridgmans = bridgman_function(ratios)
#     triax_fit = triaxs = triax_function(ratios)


